/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("dojoe/ssd/quickedit/MapMappings", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojoe/ssd/quickedit/DefaultMapper"
], function (declare, lang, on, DefaultMapper) {
     return declare("dojoe.ssd.quickedit.MapMappings", [DefaultMapper], {            
         columnTypeMap: null,
         
         statusSelect: null,
         tooltipSelect: null,
         labelSelect: null,
         imageSelect: null,
         
         defaults: {
             MARKERLAT:     "UIMapMarkerLat", 
             MARKERLON:     "UIMapMarkerLon", 
             MARKERSTATUS:  "UIMapMarkerStatus", 
             MARKERTOOLTIP: "UIMapMarkerTooltip", 
             MARKERMINZOOM: "UIMapMarkerMinZoom",
             MARKERMAXZOOM: "UIMapMarkerMaxZoom", 
             MARKERZOOMTO:  "UIMapMarkerZoomTo", 
             MARKERLABEL:   "UIMapMarkerLabel",
             MARKERIMAGE:   "UIMapMarkerImage"
         },         
             
         createMappingGrid: function () {
             this.inherited(arguments);
             
             this._buildColumnTypeMap();
             
             //set default values for mappings
             for(var i = 0; i < this.mappingGrid_data.itemsUI.length; i++) {
                 var itemUI = this.mappingGrid_data.itemsUI[i];
                 
                 switch(itemUI.item_id) {
                     case this.defaults.MARKERSTATUS:
                         this.statusSelect = itemUI;
                         on(this.statusSelect.item_value, 'change', lang.hitch(this, this.onStatusSelectChange));
                         break;
                     case this.defaults.MARKERTOOLTIP:
                         this.tooltipSelect = itemUI;
                         break;
                     case this.defaults.MARKERLABEL:
                         this.labelSelect = itemUI;
                         break;
                     case this.defaults.MARKERIMAGE:
                         this.imageSelect = itemUI;
                         break;
                 }
                 
                 this._setDefaultValue(itemUI);  
             }
         },
         
         onStatusSelectChange: function(evt) {
             var newColId = this.statusSelect.item_value.valueNode.value;
             var colType = this.columnTypeMap[newColId];
             
             if(colType === "status") {
                 this._displaySelect(this.tooltipSelect.item_value, false);
                 this._displaySelect(this.labelSelect.item_value, false);
                 this._displaySelect(this.imageSelect.item_value, false);
             } else {
                 this._displaySelect(this.tooltipSelect.item_value, true);
                 this._displaySelect(this.labelSelect.item_value, true);
                 this._displaySelect(this.imageSelect.item_value, true);
             }
         },
         
         _displaySelect: function(select, show) {             
             if(show) {
                 dojo.style(select.getParent().domNode, "display", "table-row");
                 this._setDefaultValue(select);
             } else {
                 dojo.style(select.getParent().domNode, "display", "none");
                 this._setComboBoxValue(select, "None");
             }
         },
         
         _setDefaultValue: function(itemUI) {
             if(!this._hasMappedValue(itemUI.item_id)) {
                 for(var key in this.defaults) {
                     if(itemUI.item_id === this.defaults[key]) {
                         this._setComboBoxValue(itemUI.item_value, itemUI.item_id);
                         break;
                     }                     
                 }                 
             }               
         },
         
         _hasMappedValue: function(item_id) {
             for(var i = 0; i < dashletConfig.attrNameArr.length; i++) {
                 var attrName = dashletConfig.attrNameArr[i].replace("qe_attr_", "");
                 
                 if(item_id === attrName) {
                     return true;
                 }
             }             
             return false;
         },
         
         _setComboBoxValue: function(item, value) {
             if(!value || typeof(value) == 'undefined') {
                 value = "None";
             }
             
             for(var i = 0; i < item.params.store.data.length; i++) {
                 var data = item.params.store.data[i];
                 
                 if(data.value === value) {
                     item.setValue(value);
                     break;
                 }
                 
                 //Make sure the box is set
                 if(i === item.params.store.data.length - 1) {
                     item.setValue("None");
                 }
             }             
         },
         
         _buildColumnTypeMap: function() {
             this.columnTypeMap = new Object();
             for(var i = 0; i < this.columns.length; i++) {
                 var col = this.columns[i]._col;
                 
                 this.columnTypeMap[col.id] = col.valueType;
             }
         }         
     });
});
     