// wrapped by build app
define("ibm/tivoli/navmodel/EventRouter", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.EventRouter"); 

dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.EventRouter._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.EventRouter");

////////////////////////////////////
// EventRouter
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.EventRouter", null, {
	// private members
	_args: null,
	_initialized: false,
	_globalObj: null,
	_trace: ibm.tivoli.navmodel.EventRouter._trace,
	
	/*=====
	constructor_kwArgs: function(eventDestination, timeout, pollDelay, pollErrorDelay, contextRoot) {
		// eventDestination: String
		//		destination to listen to (i.e. provided by Dataset)
		// timeout: int?
		//		number of seconds to wait for events  (optional)
		// pollDelay: int?
		//		number of ms to wait in between poll (optional)
		// pollErrorDelay: int?
		//		number of ms to wait after an error before next poll  (optional)
		// contextRoot: String?
		//		context root for requests to event servlet  (optional)		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs EventRouter object.
		this._args = kwArgs;
		
		// determine the document that we want to anchor the javascript under
		try {
			this._globalObj = ISOLATION_INIFRAME ? parent : window;
		}
		catch(e) {
			//Ramachandran - 30-Aug-13 - 38398: Show warning message when using TWL dynamic update widget in web widget
			//Modified from parent to window reference
			this._globalObj = window;
		}		
		
		// load javascript if we haven't already				
		this._loadEventRouterJavascript();	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	/*=====
	addListener_event: function(event){},
	=====*/

	addListener: function(/*String*/ eventId, /*addListener_event*/ eventCallback) {
		// summary:
		//		Adds a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		// eventCallback:
		//		callback function to invoke when events are received
		if (this._initialized) {
			if (this._trace.isFinest()) {
				this._trace.finest("adding listener for eventId=" + eventId, "addListener");
			}
			
			this._getGlobalEventRouter().addListener(this, eventId, eventCallback);
		}
		else {
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "addListener");
			}
					
			setTimeout(dojo.hitch(this, function() {this.addListener(eventId, eventCallback);}), 50);
		}
	},

	removeListener: function(/*String*/ eventId) {
		// summary:
		//		Removes a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		if (this._initialized) {
			return this._getGlobalEventRouter().removeListener(this, eventId);
		}
		
		return 0;
	},

	ping: function() {
		// summary:
		//		Check if the event router is still alive
		console.assert(true);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_init: function() {	
		// summary:
		//		Initializes the event router
		var globalEventRouter = this._getGlobalEventRouter();
		
		if (globalEventRouter) {
			if (!globalEventRouter.isInitialized()) {
				if (this._trace.isFinest()) {
					this._trace.finest("initializing global event router", "_init");
				}
			
				globalEventRouter.init(this._args);
			}
			
			this._initialized = true;
		}
		else {		
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "_init");
			}
					
			setTimeout(dojo.hitch(this, function() {this._init();}), 50);
		}	
	},

	_loadEventRouterJavascript: function() {
		// summary:
		//		Load event_router.js using script tag
		var id = "ibm_tivoli_navmodel_event_router";
		var script = this._globalObj.document.getElementById(id);
		if (!script && !this._getGlobalEventRouter()) {
			if (this._trace.isFinest()) {
				this._trace.finest("Inserting event_router.js", "_loadEventRouterJavascript");
			}
			
			// so we can use inside onreadystatechange
			var that = this;
			
			//62108: TWL perf - Fix the Aborted event_router.js Get
			//Bobby: 21Jan2013, let's fix the event router path early, before setting to script tag itself
			var eventRouterSrc = require.toUrl("ibm/tivoli/navmodel")+ "/event_router.js";
			// Check if presence of cacheBust messed up the path to the .js file
			//console.info("eventRouterSrc:"+eventRouterSrc);
			if (dojo.config.cacheBust) {
			    eventRouterSrc = eventRouterSrc.slice(0,eventRouterSrc.indexOf("?")) + "/event_router.js?" + dojo.config.cacheBust;
			}			
			//console.info("eventRouterSrc:"+eventRouterSrc);			
			
			// create the script tag
			script = this._globalObj.document.createElement('script');
			script.id = id;
			script.type = "text/javascript";
			// Dojo 1.7 migration
			// script.src = dojo.moduleUrl("ibm.tivoli.navmodel", "event_router.js").uri;
			
			/*
			62108: TWL perf - Fix the Aborted event_router.js Get
			Bobby: 21Jan2013, changing after setting script.src is creating issues with IE
			script.src = require.toUrl("ibm/tivoli/navmodel")+ "/event_router.js";
			// Check if presence of cacheBust messed up the path to the .js file
			if (dojo.config.cacheBust) {
			    script.src = script.src.slice(0,script.src.indexOf("?")) + "/event_router.js?" + dojo.config.cacheBust;
			}*/
			
			script.src = eventRouterSrc;
			script.onload = dojo.hitch(this, this._init);
			script.onreadystatechange = function () { 
				if (this.readyState == 'loaded' || this.readyState == 'complete') {
					that._init();				
				} 
			};
			
			// insert the script tag		
			this._globalObj.document.body.appendChild(script);			
		}
		// script has loaded or in the process of loading
		else {	
			if (this._trace.isFinest()) {
				this._trace.finest("event_router.js is already loaded", "_loadEventRouterJavascript");
			}
			this._init();				
		}		
	},

	_getGlobalEventRouter: function() {
		// summary:
		//		Returns instance of global event router; depending on how javascript was loaded
		var eventRouter = this._globalObj.ibm_tivoli_navmodel_EventRouter;		
		if (!eventRouter) {
			try { 
				eventRouter = ibm_tivoli_navmodel_EventRouter;
			}
			catch (e) {
				// event_router.js has not been loaded
			}
		}
		
		return eventRouter;
	}
});


ibm.tivoli.navmodel.EventRouter.getPageId = function() {	
	// determine the document that we want to anchor the javascript under
	var globalObj = null;
	try {
		globalObj = ISOLATION_INIFRAME ? parent : window;		
	}
	catch(e) {
		//Ramachandran - 30-Aug-13 - 38398: Show warning message when using TWL dynamic update widget in web widget
		//Modified from parent to window reference
		globalObj = window;
	}
	
	if (!globalObj.ibm_tivoli_navmodel_EventRouter_pageId) {
		var now = new Date();
		globalObj.ibm_tivoli_navmodel_EventRouter_pageId = now.getTime().toString();
	}
	
	return globalObj.ibm_tivoli_navmodel_EventRouter_pageId;
};
});
