// wrapped by build app
define("ibm/tivoli/tip/dijit/TIPContextMenuProvider", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
/**
 *  You provide a context menu provider to do things like populate the context menu with
 *  items, and to handle the actions when one of those items are clicked.
 *  You provide:
 *  - getMenuItemsCallback (required)   (optional alternative: override getMenuItems)
 *      Parameters: (context)
 *                 context (that was passed on the contextMenuOpen call (in your code)
 *
 *      Summary: This callback provides the menu items to be inserted into the context menu.          
 *      The following defines the callback that is used to return menu items to appear
 *      in the list.  This method should return JSON data in the form:
 *       [ 
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon", 
 *            "actionID": "testActionID" 
 *       },
 *       { 
 *            "actionID": "_separator_" 
 *       },
 *       { 
 *            "label": "submenu", 
 *            "iconClass": "tipMenuIcon", 
 *            "submenu_items": 
 *            [ 
 *             { 
 *                  "label": "Delete", 
 *                 "iconClass": "tipMenuIcon tipDeleteIcon", 
 *                 "actionID": "testDeleteActionID" 
 *             }
 *            ] 
 *       } 
 *       ]
 *
 *  - getMenuCommandCallback (optional)  (optional alternative: override getMenuCommand)
 *      Parameters: (commandName, context, menuItem)
 *                 commandName - the command name of the context menu item.  Provided on the
 *                               'actionID' item above. 
 *                 context - (that was passed on the contextMenuOpen call (in your code)
 *                 menuItem - the actual menu item object.
 *      This callback provides the function needed to process a menu item click.
 *      The response depends largely on the action you wish to take.  A sample response:
 *
 *      {
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *      }
 *      Launch types include: JavaScript, WEB_URL, CLIENT_EVENT, PORTLET, PORTAL_PAGE, NOOP
 *
 *      The actions listed above may be included in the original information returned by 
 *      the getMenuItemsCallback.  If these actions are included, you need not specify
 *      a getMenuCommandCallback, and the provider will process these actions for you
 *      automatically.  For example:
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon",
 *            "actionID": "testActionID", 
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *       },
 */
 
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProvider");


dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProvider", null,
{
   iscPortletNamespace: null,   // the namespace of this UI's portlet
                                // (must be set to enable launching of portlet or portlet page from a UI in
                                // a non-iWidget portlet)
   
   iscPortletInstanceID: null,  // the instance ID of this UI's portlet (also called "oid" in portletData)
                                // (must be set to enable launching of portlet or portlet page from a UI in
                                // an iWidget)
   
   is_iWidget: false,  // indicates if this menu provider is being used on a UI in an iWidget
   
   eventFormat: "mixed", // Event context format: "new"=new context, "old"=old context, "mixed"=new and old
   
   iWidgetIDPrefix: "_iw",  // prefix needed for iscPortletInstanceID if UI is an iWidget

   getMenuItemsCallback: null,
   
   getMenuCommandCallback: null,
   
   launchNewWindowOnNullId :true,

   menus_: null, // List of menus saved from getMenuItems

   constructor: function( args ) 
   {
      // summary:
      //      The base class for all providers that populate the TIPContextMenu
      dojo.mixin( this, args );

      this.id_ = "1_";
   },

   getMenuItems: function(context)
   {
      this.menus_ = this.getMenuItemsCallback(context);
      return this.menus_;
   },

   getItem: function(/*String*/ id) 
   {
      var submenus = [];
      var currentMenu = null;
      var x = null;
      submenus.push(this.menus_);
      
      currentMenu = submenus.pop();
      while (currentMenu)
      {
         x = currentMenu.pop();
         while (x)
         {
            if (x) 
            {
               if (x.submenu_items)
               {
                  submenus.push(x.submenu_items);
               }
               else
               {
                  if (x.id == id || x.actionID == id)
                  {
                     return x;
                  }
               }
            }
            x = currentMenu.pop();
         }
         currentMenu = submenus.pop();
      }
   },
   
   isArray: function (obj) 
   {
      if (obj.constructor.toString().indexOf("Array") == -1) 
      {
         return false;
      } else {
         return true;      
      }
   },

   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
      }
      else
      {
         result = this.getItem(commandName);
      }

      // console.debug("------------ handleMenuAction result",result);
      this.handleMenuAction(result);
   },

   handleMenuAction: function (data, ioArgs) 
   {
      // console.info ("handleMenuAction: ");
      // console.info (data);

      if (data.launchType == "JavaScript") {
         this.handleJavascript (data);

      } else if (data.launchType == "WEB_URL") {
         this.launchURL (data);

      } else if (data.launchType == "CLIENT_EVENT") {
         this.sendEvent (data);

      } else if (data.launchType == "PORTLET") {
         this.launchPortlet (data);

      } else if (data.launchType == "PORTAL_PAGE") {
         this.launchPage(data);

      } else if (data.launchType == "NOOP") {
         // Do nothing in the case of a no operation type

      } else {
         console.error ("unsupported launchtype: " + data.launchType);
      }
   },

   handleJavascript: function (data) 
   {
      // console.log ("enter handleJavascript");
      var fnc = new Function ("eval (\"" + data.uri + "\");");
      fnc();
   },

   launchURL: function(data) 
   {
	   console.log ("enter launchURL id ");
		  console.info("browserwindowid :"+data.browserwindowid + " launchNewWindowOnNullId :"+this.launchNewWindowOnNullId);
		 var width  = 300;
		 var height = 200;
		 var left   = (screen.width  )/2;
		 var top    = (screen.height)/2;
		 var params = 'top='+top+', left='+left +', resizable=yes';
		 /*
		  * 53793
		  * If the user wants open new page in a new tab,
		  * this params will prevent Firefox to do that.
		  * Firefox will always open a new window if params
		  * is not null. Setting params to '' will allow FF
		  * open a new tab.
		  */
		 params = '';
		
		 if(data.browserwindowid != null) {
		   window.open(data.uri,data.browserwindowid, params);
		 }
		 else {
		   if(this.launchNewWindowOnNullId == true) {
		      window.open(data.uri,"_blank", params);
		   }
		   else {
		      window.open(data.uri,"_self", params);
		   
		   }
		   
		 }
   },

   launchPortlet: function (data) 
   {
	   var portletEvent = {
                  name: "http://ibm.com/isclite#launchPortlet",
                  portletName: data.portletName,
                  portletApplicationID: data.portletAppUID
               };
	      
	   if (data.parameters)
	   {
		   for (var j=0; j<data.parameters.length; j++)
		   {
			   portletEvent[data.parameters[j].name] = data.parameters[j].value.toString();
		   }
	   }
	      
	   try
	   {
		   EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(portletEvent));
	   } catch (e) {
		   console.error ("Caught javascript exception in TIPContextMenuProvider.launchPortlet: " + e);
		   alert (e);
	   }		  
   },
   
   launchPage: function (data) 
   {
	  if (this.is_iWidget)
	  {
		  // Get the TIP iWidget event service
		  var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
		  var eventArray = new Array();
		  var eventData = {};
		  eventData.targetEvent = (("event" in data) && (data.event != null)) ? data.event : "http://ibm.com/isclite#launchPage"; 

		  // Set up the launch parameters as the payload
		  eventData.payload = {};
		  eventData.payloadType = "String";
		  if (data.parameters)
		  {
			  for (var k=0; k<data.parameters.length; k++)
			  {
				  eventData.payload[data.parameters[k].name] = data.parameters[k].value.toString();
			  }
		  }
		  eventData.payload = this.formatOutboundEvent(eventData.payload);
		  eventArray.push(eventData);
		  // Make sure the portletInstanceID has the iWidget prefix
		  var portletInstanceID = this.iscPortletInstanceID;
		  if (portletInstanceID && (portletInstanceID.indexOf(this.iWidgetIDPrefix) == 0))
		  {
			  portletInstanceID = this.iWidgetIDPrefix + portletInstanceID;
		  }
		  try 
		  {
			  eventService.broadcastEvents(eventArray, this.iscPortletInstanceID, data.portletPageID);			  
		  } catch (ex) {
			  console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
			  alert (ex);
		  }		  
	  }
	  else
	  {
		  var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: data.portletPageID,
                         pageInstanceRef: null,
                         switchPage: true
          };
		  
		  if (data.parameters)
		  {
			  for (var k=0; k<data.parameters.length; k++)
			  {
				  pageEvent[data.parameters[k].name] = data.parameters[k].value.toString();
			  }
		  }
		  
		  try
		  {
			  EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(pageEvent));
		  } catch (ex) {
			  console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
			  alert (ex);
		  }
	  }
   },

   /* Formats an iWidget payload or porlet event.  The input parameter will be the   */
   /* entire event for portlets, and it will be the payload for iwidgets.            */
   /* This method will format the context according to the flag set in this provider */
   formatOutboundEvent: function(payload)
   {
      var newPayload=payload;
      if (this.eventFormat != "old"){ // CMS builds old context by default
         var util = new ibm.tivoli.tip.dojo.EventUtil();
         if (this.eventFormat == "mixed"){
            util.addNewContext(newPayload);
         } else if (this.eventFormat == "new"){
            newPayload={};
            if (typeof payload.name != "undefined"){ // Portlets have the event name coded in
               newPayload.name = payload.name;
            }
            if (typeof payload.source != "undefined"){ // Portlets have the event name coded in
               newPayload.source = payload.source;
            }
            newPayload.payload={};
            util.addNewContext(payload,newPayload.payload);
         }
      }
      return newPayload;
   },
   
   sendEvent: function (data) 
   {
   
   	  var standardEvent = this.createCMSEventCallback ? this.createCMSEventCallback(data) : null;   	  
   
      var parms = data.parameters;
	  if (this.is_iWidget)
	  {
		  // Get the TIP iWidget event service.  aasinha : the below service would not be required. This refers to 
		  // another overloaded fireEvent function which needs source and taget widget IDs.
		  // var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
		  var iwId=this.iWidgetIDPrefix + this.iscPortletInstanceID;
		  var iWidgetContainer = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel();
		  var widgetObj = iWidgetContainer.getWidgetById(iwId);
		  if(widgetObj == null) {
			widgetObj = iWBridge.findIframedWidget(iwId);
		  }
		  var widget_iContext = widgetObj.iScope.iContext;
		  if(!standardEvent){
		  	var payload = {};
		  	// Set up the event parameters as the payload
		  	if (parms)
		  	{
			  	for (var i=0; i<parms.length; i++)
			  	{
				  	payload[parms[i].name] = parms[i].value.toString();
			  	}
		  	}
			//	eventService.fireEvent(data.event, "String", this.formatOutboundEvent(payload));  aasinha1 : this call goes to another method which needs source and target widget.
			widget_iContext.iEvents.fireEvent(data.event, "String", this.formatOutboundEvent(payload));
		  }else{
		  	//	eventService.fireEvent(standardEvent.name, "String", this.formatOutboundEvent(standardEvent.payload));
			widget_iContext.iEvents.fireEvent(standardEvent.name, "String", this.formatOutboundEvent(standardEvent.payload));
		  }			  
	  }
	  else
	  {
	  	if(!standardEvent){
		  var event = {
				  name: data.event
                  };

		  if (parms)
		  {
			  for (var i=0; i<parms.length; i++) 
			  {
				  event[parms[i].name] = parms[i].value.toString();
			  }
		  }		  
		  // console.log ("TIPContextMenu: sending event: ");
		  // console.log (event);
		  EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(event));
		  }else{
		  	EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(standardEvent));
		  }
	  }
   },

   destroy: function()
   {
   		this.inherited (arguments);
      // Placeholder: for overrides.  Exploiters should call this as best practice. 
   }
      
});
});
