/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
		"ibm/usmi/core/framework/frameworkdata", [
			"dojo/_base/declare",
			"dojo/data/ItemFileReadStore",
			"dojo/topic",
			"ibm/tivoli/navmodel/Dataset",
			"dojo/store/JsonRest",
			"ibm/usmi/util/DefaultMessageHandler",
			"ibm/usmi/widgets/util/Constants",
			"ibm/tivoli/navmodel/Selection",
			"ibm/usmi/core/utils/Logger",
			"dojo/_base/lang",
			"dojo/i18n!ibm/usmi/nls/frameworkResources"			
		],
		function (declare, itemFileReadStore, topic, Dataset, JsonRest, DefaultMessageHandler, Constants, Selection,
				Logger, djLang, MRI) {
			return declare(
					null,
					{
						loggerTask : undefined,
						dataset : undefined,
						_frameworkdataLogger : new Logger("ibm.usmi.console.framework.core", "frameworkdata"),

						constructor : function () {
							this.dataset = new Dataset({
								onComplete : dojo.hitch(this, function (dataset, kwArgs) {
									dataset.getItems({
										onComplete : dojo.hitch(this, this.getItemsComplete),
										properties : [ "all"
										]
									});

									dataset.getTasks({
										onComplete : dojo.hitch(this, this.getTasksComplete),
										selection : new Selection({
											version : dataset.getVersion()
										})
									});
								}),
								onError : function (error, exception, ioArgs) {
									var msgHandler = new DefaultMessageHandler({});
									msgHandler.displayException(exception);
								},
								baseUrl : Constants.BASE_REST_URL,
								provider : "USMI",
								datasource : "",
								dataset : "FRAMEWORK",
								parameters : {},
								defaultProperties : [ "all"
								]
							});
						},

						destroy : function () {
							if (this.dataset) {
								this.dataset.destroy();
							}
						},

						getItemsComplete : function (items) {
							var singleNode, sessionId, pName, uName, pVersion, sslCertInfo, infoCenterURL, alertStatusPodPermission, serverIsNISTCompliant, jobStatusPodPermission, storageKey = "usmiLocalStorageKey", methodname = "getItemsComplete", productKey = "brandedProductKey", productVersionKey = "brandedProductVersionKey";
							if (items && items[0]) {
								singleNode = items[0];
								sessionId = singleNode.getProperty("SESSIONID").getValue();
								pName = singleNode.getProperty("PRODUCTNAME").getValue();
								uName = singleNode.getProperty("USERNAME").getValue();
								alertStatusPodPermission = singleNode.getProperty("ALERTPERMISSION").getValue();
								serverIsNISTCompliant = singleNode.getProperty("NISTCOMPLIANT").getValue();
								jobStatusPodPermission = singleNode.getProperty("JOBPERMISSION").getValue();
                                if(singleNode.getProperty("PRODUCTVERSION")) {
                                    pVersion = singleNode.getProperty("PRODUCTVERSION").getValue();
                                }								
								if(singleNode.getProperty("SSLCERTINFO")){
									sslCertInfo = singleNode.getProperty("SSLCERTINFO").getValue();
								} else {
									sslCertInfo = "";
								}
								if(singleNode.getProperty("INFOCENTERURL") && singleNode.getProperty("INFOCENTERURL").getValue() !== ""){
									infoCenterURL = singleNode.getProperty("INFOCENTERURL").getValue();
								} else {
									infoCenterURL = "";
								}
								
							} else {
								gInvalidUser = true;
								uName = localStorage.getItem("loginUser");
								pName = localStorage.getItem("loginProduct");
								sessionId = localStorage.getItem("loginSession");
								pVersion = localStorage.getItem("loginProductVersion");
							}					

							//check local storage usmiLocalStorageKey==current
							//if not, localstorage.clear()
							this._frameworkdataLogger.finer("checking local storage", methodname);
							this._frameworkdataLogger.finer("localStorage.key: " + localStorage.getItem(storageKey),
									methodname);
							if (localStorage.getItem(storageKey) !== sessionId) {
								this._frameworkdataLogger.finer("didn't equal the key", methodname);
								//store new value in localStorage
								localStorage.clear();
								localStorage.setItem(storageKey, sessionId);
								localStorage.setItem("loginUser", uName);
								localStorage.setItem("loginProduct", pName);
								localStorage.setItem("loginSession", sessionId);
								localStorage.setItem("loginProductVersion", pVersion);
							} else {
								this._frameworkdataLogger.finer("local storage key equals the session id already",
										methodname);
							}
							localStorage.setItem("sslCertInfo", sslCertInfo);
							localStorage.setItem("infoCenterURL", infoCenterURL);
							localStorage.setItem("serverIsNISTCompliant", serverIsNISTCompliant);							
							localStorage.setItem(productKey, pName);
							localStorage.setItem(productVersionKey, pVersion);
							topic.publish("curi-framework-ready", [
								pName,
								uName,
								sessionId,
								alertStatusPodPermission,
								jobStatusPodPermission,
							    pVersion
							]);

						},
						getTasksComplete : function (tasks) {
							//loop and look so we can handle future tasks here
							for ( var a = 0, numitems = tasks.length; a < numitems; a += 1) {
								//TODO: make this LOGGER_TASK
								if (tasks[a].getId() === "LOGGER_TASK") {
									this.loggerTask = tasks[a];
									topic.subscribe(Constants.TOPIC_LOG_JS_TO_SERVER, dojo
											.hitch(this, this.logToServer));
								}
							}

						},
						logToServer : function (logInfo) {
							var methodName = "logToServer";
							this._frameworkdataLogger.entering(methodName);
							this._frameworkdataLogger.finer(logInfo, methodName);
							var taskExecuteComplete = function () {
								this._frameworkdataLogger.finer("log task executed successfully!", methodName);
							};
							var taskExecuteFailed = function () {
								this._frameworkdataLogger.warn("log task execution failed!", methodName);
							};

							this.loggerTask.execute({
								onComplete : dojo.hitch(this, taskExecuteComplete),
								onError : dojo.hitch(this, taskExecuteFailed),
								selection : new Selection({
								version : this.dataset.getVersion()
								}),
								selectionParms : logInfo
							});
						}
					});
		});
