/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/menus/Menu", [
	"require",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/_base/lang",
	"idx/widget/MenuBar",
	"ibm/usmi/statuspod/AlertStatusPod",
	"ibm/usmi/statuspod/JobStatusPod",
	"dijit/PopupMenuBarItem"
], function (require, dojo, declare, WidgetBase, lang, MenuBar, AlertStatusPod,
		JobStatusPod, PopupMenuBarItem) {
	return declare([
		WidgetBase,
		MenuBar
	], {

		showProblemStatusPod : false,

		showJobStatusPod : false,
		
		//Contains the information for the user menu with fields "displayName" for the user's
		//name and "menu" for the popup menu
		userMenu : null,
		
		helpMenu : null,

		baseClass : "menus.Menu",

		constructor : function (args) {
			dojo.safeMixin(this, args);

		},

		postCreate : function () {
			this.inherited(arguments);
			if (this.showProblemStatusPod) {
				var problemMenuBarItem = new AlertStatusPod({
					id : "alertStatusPod"
				});
				this.addChild(problemMenuBarItem);
			}
			if (this.showJobStatusPod) {
				var jobMenuBarItem = new JobStatusPod({
					id : "jobStatusPod"
				});
				this.addChild(jobMenuBarItem);
			}
			if (this.userMenu) {
				this.addChild(new PopupMenuBarItem({ label : "<span class='idxHeaderUserMenuBarItemLabel'></span>" + this.userMenu.displayName, popup: this.userMenu.menu}));
			}
			if (this.helpMenu) {
				
				this.addChild(new PopupMenuBarItem({ label : "<span class='idxHeaderHelpIcon'></span>", popup: this.helpMenu}));
			}
		},
		onItemHover : function(/*MenuItem*/ item){
			var delay = 500;
			// In TWL 1.1.0.3 _MenuBase::onItemHover they call setTimeout passing the method and the popupDelay time.  
			// In TWL 3.1.1.0 they do not use the setTimeout any longer.  Here I'm re-implementing the old way			
			this._hovered = item;
			var $this = this;
			// I also discovered that IE doesn't work using the lang.hitch. It losses the object parameters. They become undefined 
			// passed to the called method.  This is because the setTimeout is referencing the window and not the "this" object.
			// To fix this issue I've implemented the setTimeout as an anonymous function. You have to save the "this" object so
			// that the anonymous function can find the method and its parameters.  Unfortunately the anonymous function didn't 
			// work in Chrome.			
			if(dojo.isIE){
				setTimeout(function() {$this._onIMenuHover(item, arguments);}, delay);	
			} else {
				setTimeout(lang.hitch (this,"_onIMenuHover"), delay, item, arguments);				
			}
		},
		
		_onIMenuHover: function(/*MenuItem*/ item, /*MenuBar arguments*/ args){			
			
		
			// If the mouse is moving between all menu items, check if the one the mouse is hovering on is the current one otherwise cancel
			if(this._hovered !== item ) return;
			// if the mouse is moving between a menu item but is not hovering for at least 1/2 second then don't display the menu
			if(!item.hovering) return;
			
			var childMenuItems;
			// 204123 (and 204797 for Home menu bar (Status/Jobs/Userid/Help)) Mouse hover on Mega menu item activates menu right away. 
			// There needs to be a delay to give the user time to move to another item
			// this.inherited() calls js/idx/widget/_MenuOpenOnHoverMixin.js onItemHover function.  _MenuOpenOnHoverMixin calls
			// js/dijit/MenuBar.js focusChild which popups the menu immediately.  This is new with TWL 3.1.1.0.  That is, TWL removed
			// the timer delay for some reason. To work around this problem I'm simply adding a sleep before calling the inherited()
			// Unfortunately, the code to cancel the hovering is also not available.  Above I'm checking if the item is still 
			// hovering.  If not, return.
			// Also, I found that TWL 3.1.1.0 also checks for IE 11 and if this is the browser it doesn't call this.focusChild. 
			if(dojo.isIE){				
				this.focusChild(item);
                        }
			
			this.inherited(args);
		}
	});
});
