/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

/**
 * NextGenUIHelpMenu is a specialized menu for help links and extends from the
 * oneui menu
 */
define("ibm/usmi/menus/NextGenUIHelpMenu", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/dom-construct",
	"dijit/registry",
	"dojo/query",
	"dijit/MenuItem",
	"idx/widget/Menu",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/menus/NextGenUIMenuUtils",
	"ibm/usmi/core/utils/Logger",
	"dojo/topic"
], function (dojo, declare, WidgetBase, domConstruct, registry, query, MenuItem, Menu, DataStore,
		_UsmiBrowserPersistence, MRI, NextGenUIMenuUtils, Logger, topic) {
	return declare([
		WidgetBase,
		Menu,
		_UsmiBrowserPersistence
	], {

		_helpMenu : null,
		_dataStore : null,
		_nextGenUIMenuUtils : new NextGenUIMenuUtils({}),
		_nextGenUIHelpMenuLogger : new Logger("ibm.usmi.menus", "NextGenUIHelpMenu"),
		_PERSISTENCE_HELP_MENU_ID : "ibm.usmi.menus.NextGenUIHelpMenu.contents",

		constructor : function (args) {
			dojo.safeMixin(this, args);
		},

		postCreate : function () {
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.entering("", "postCreate");
			}
			this.inherited(arguments);

			this._helpMenu = this.getData(this._PERSISTENCE_HELP_MENU_ID);
			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info(
						"Checked session storage for help menu, value is: " + this._helpMenu, "postCreate");
			}
			if (this._helpMenu) {
				if (this._nextGenUIHelpMenuLogger.isInfo()) {
					this._nextGenUIHelpMenuLogger.info("Posting topic that help menu is loaded", "postCreate");
				}
				topic.publish("nextgen-ui-helpmenu-loaded", null);
			} else {
				if (this._nextGenUIHelpMenuLogger.isInfo()) {
					this._nextGenUIHelpMenuLogger.info(
							"Help menu hasn't been stored yet, need to pull contents from server.", "postCreate");
				}
				this._dataStore = new DataStore({
					datasetId : 'HeaderMegaMenuCollection',
					homeParms : null,
					getItems : true,
					retrieveItemProperties : true,
					getTasks : false,
					initReadyCallback : dojo.hitch(this, this.initializeDataStores)
				});
			}
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.exiting("", "postCreate");
			}
		},

		initializeDataStores : function () {
			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info(
						"Help menu data retrieved from server. Converting CURI data into the help menu object",
						"initializeDataStores");
			}
			this._convertMenuCollectionToHelpMenuObject();
			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info("Saving help menu data to the session", "initializeDataStores");
			}
			this.setData(this._PERSISTENCE_HELP_MENU_ID, this._helpMenu);

			// after we process datastore items, and persist in storage clean up the datastore as it is no
			// no longer needed.
			this.destroyDataStore();
		},
		
		destroyDataStore : function () {
			// clean up the datastore
			if (this._dataStore) {
				this._dataStore.getDataset().removeEventListener(this);
				this._dataStore.destroy();
				this._dataStore = null;
			}
			
			this.inherited(arguments);
		},

		/**
		 * converts a collection into a help menu object for easy storage in
		 * session persistence
		 */
		_convertMenuCollectionToHelpMenuObject : function () {
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.entering("", "_convertMenuCollectionToHelpMenuObject");
			}
			var items, sortedItems, itemType, menuId, i;
			sortedItems = [];
			items = this._dataStore.getItems();
			this._helpMenu = [];
			for (i = 0; i < items.length; i++) {
				sortedItems[items[i].getProperty('ORDER_COLUMN').getValue()] = items[i];
			}

			dojo.forEach(sortedItems, dojo.hitch(this, function (entry, i) {
				itemType = entry.getProperty('TYPE_COLUMN').getValue();
				menuId = entry.getProperty('MENU_ID_COLUMN').getValue();
				if (itemType === 'menuItem') {
					if (menuId === 'HelpMenu') {
						this._helpMenu.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"task" : entry.getProperty('TASK_COLUMN').getValue(),
							"id" : menuId
						});
					}
				} else if (itemType === 'menuHeading') {
					if (menuId === 'HelpMenu') {
						this._helpMenu.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue()
						});
					}
				}

			}));

			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info("Posting topic that help menu is loaded",
						"_convertMenuCollectionToHelpMenuObject");
			}
			topic.publish("nextgen-ui-helpmenu-loaded", null);
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.exiting("", "_convertMenuCollectionToHelpMenuObject");
			}
		},

		/**
		 * creates the help menu from the help menu object
		 */
		createHelpMenu : function () {
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.entering("", "_createHelpMenu");
			}
			var i;
			if (this._helpMenu) {
				for (i = 0; i < this._helpMenu.length; i++) {
					if (this._helpMenu[i].type === 'menuItem') {
						this.addChild(this._nextGenUIMenuUtils.createMenuItem(this._helpMenu[i].title,
								this._helpMenu[i].column, this._helpMenu[i].task, MRI));
					} else if (this._helpMenu[i].type === 'menuHeading') {
						this.addChild(this._nextGenUIMenuUtils.createMenuHeading(this._helpMenu[i].title,
								this._helpMenu[i].column, MRI));
					}
				}
			} else {
				if (this._nextGenUIHelpMenuLogger.isWarn()) {
					this._nextGenUIHelpMenuLogger.warn("Help menu object is null", "_createHelpMenu");
				}
			}
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.exiting("", "_createHelpMenu");
			}
		}
	});

});