/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/StatusPodMenuItem", [
    "dojo/_base/kernel",
    "dojo/_base/declare",
    "dijit/MenuItem",
    "ibm/usmi/core/utils/Logger"
], function(dojo, declare, MenuItem, Logger) {
    return declare([ MenuItem
    ], {
        // summary:
        //      This provides functions useful to StatusPodMenuItem

        hoverTip: null,

        //object id of resource for this menu item
        objectId: null,

        _logger: new Logger("ibm.usmi.statuspod", "StatusPodMenuItem"),

        _onBlur: function() {
            // summary:
            //      the callback method to menu item blur event
            this.inherited(arguments);
            this._setSelected(false);
        },

        _onHover: function() {
            // summary:
            //      the callback method to menu item hover event
            if (this.hoverTip) {
                this.hoverTip.show();
            }
        },

        _onUnhover: function() {
            // summary:
            //     the callback method to menu item unHover event
            this.inherited(arguments);
            if (this.hoverTip) {
                this.hoverTip.hide();
            }
        },

        setHoverTip: function(myHoverTip) {
            // summary:
            //     set the hover tooltip
            this.hoverTip = myHoverTip;
        },
		
		// jdwald:  a bit of a hack to ensure the statuspod fly-over text appears as required.
		_setSelected: function(selected){
			if (this.domNode) {
				this.inherited(arguments);
			}
			
			if (selected)			
				this._onHover();
			else
				this._onUnhover();		
		},
		
        close: function() {
            // summary:
            //     close the menu item 
            this._onUnhover();
        },

        destroy: function() {
            // summary:
            //     destroy the menu item 
            this.inherited(arguments);
            if (this.hoverTip) {
                this.hoverTip.destroyRecursive();
                this.hoverTip = null;
            }
        }
    });
});
