/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/_StatusPodPopupMixin", [
    "dojo/_base/kernel",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "ibm/usmi/core/utils/Logger"
], function(dojo, declare, lang, array, Logger) {
    return declare(null, {
        // summary:
        //      This provides functions useful to AlertStatusPodDialogContent and JobStatusPodDialogContent
        menuItems: [],

        _logger: new Logger("ibm.usmi.statuspod", "_StatusPopupMixin"),

        closeDescendants: function() {
            // summary:
            //      close the descendants of menu
            this._closeMenuItems(this.menuItems);
        },

        _closeMenuItems: function(menuItems) {
            // summary:
            //      close the menu items
            // menuItems: StatusPodMenuItem[]
            //      the cached array for menu items
            this._logger.finest("close all descendants menu items", "_closeMenuItems");
            var items = menuItems;
            if (!items) {
                return;
            }
            array.forEach(items, lang.hitch(this, function(menuItem) {
                if (menuItem && menuItem.close) {
                    this._logger.finest("close menu item", "_closeMenuItems");
                    menuItem.close();
                }
            }));
        },

        addDescendant: function(/*StatusPodMenuItem*/menuItem, menuItemsArray) {
            // summary:
            //      add a menu item as a descendant of the menu, then we can close or destroy them later
            // menuItem: StatusPodMenuItem
            //      the menu item to be added
            // menuItemsArray: StatusPodMenuItem[]
            //      the cached array for menu items
            var items = menuItemsArray;
            if (!items) {
                items = this.menuItems;
            }
            if (!items) {
                return;
            }
            items.push(menuItem);
        },

        destroyMenuItems: function(menuItems) {
            // summary:
            //      destroy the menu items
            // menuItems: StatusPodMenuItem[]
            //      the cached array for menu items
            var items = menuItems;
            if (!items) {
                items = this.menuItems;
            }
            if (!items) {
                return;
            }
            array.forEach(items, lang.hitch(this, function(menuItem) {
                if (menuItem && menuItem.destroyRecursive) {
                    this._logger.finest("destroy menu item", "destroyMentItems");
                    menuItem.destroyRecursive();
                }
            }));

            //clear the array
            this.clearArray(items);
        },

        clearArray: function(oldArray) {
            // summary:
            //      helper method to empty an array
            if (!oldArray) {
                oldArray = [];
            } else {
                oldArray.length = 0;
            }
        }

    });
});
