/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2013, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/util/DefaultDialog", [
	"dijit/registry",
	"dijit/_WidgetBase",
	"dijit/Dialog",
	"dojo/_base/declare",
	"dojo/topic",
	"dojo/_base/lang",
	"dojo/window",
	"dojo/dom-geometry",
	"ibm/usmi/core/utils/Logger",
	"ibm/usmi/widgets/wizard/DirectorWizard",
	"dojo/on",
	"dojo/_base/connect",
	"dojo/keys"
],

function (registry, _WidgetBase,_Dialog, declare, topic, lang, winUtils, domGeometry, Logger, DirectorWizard, on, connect, keys) {

	return declare("ibm.usmi.util.DefaultDialog", [
		_Dialog
	], {


		//		Utility for logging
		_defaultDialogLogger : new Logger("ibm.usmi.util", "DefaultDialog"),
		
		title : "",
		
		// private members       
		_args : null,

		//represents the widget created by the task content
		_dialogDijit: null,
		_subscriptions: null,

		
		constructor : function (/*constructor_kwArgs*/kwArgs) {
			
			if (this._defaultDialogLogger.isFinest()) {
				this._defaultDialogLogger.entering("", "constructor");
			}
			
			this._subscriptions = [];
			
			//save the args for use below		
			this._args = {
				launchParms: "",
				menuItem: "",
				context: "",
				title: ""
			};
			lang.mixin(this._args, kwArgs);
			
			//Call superclass
			this.inherited(arguments);
			
		},
		
		postCreate: function () {
			//Call superclass
			this.inherited(arguments);
			

			//Create the internal content
			this._handleDialogLaunch();
			
			// Register for a Dialog dismissal event
			var sub = topic.subscribe("DefaultDialogDismissal", lang.hitch(this, this._close));
			this._subscriptions.push(sub);
			
			//on IE9 the ESC key is not being recogized by keypressed (which is what the 
			//underlying Dialog superclass uses to check esc on).  We will register on
			//keydown as well so that we can ensure that when ESC key is pressed we exit
			
			on(this.domNode, "keydown", lang.hitch(this, this._keypressed));
			//var keyhandle = on(this.domNode, "keydown", lang.hitch(this, this._keypressed));
			//this._subscriptions.push(keyhandle);
		},


		destroy: function () {

			//clean up any references to created widgets
			if (this._dialogDijit) {
				this._dialogDijit.destroyRecursive();
				this._dialogDijit = null;
			}
			
			//clean up subscriptions
			if (this._subscriptions) {
				for (var i = 0; i < this._subscriptions.length; i++) {
					this._subscriptions[i].remove();
				}
				this._subscriptions = null;
			}
			this.inherited(arguments);
		},
		
		_keypressed: function (e){
			
			if(e.keyCode === keys.ESCAPE){
				registry.byId(this.id).hide();
				registry.byId(this.id).destroy();
			}
		},

		//override the onClose method to also destroy the whole widget.
		_close: function () {

//			registry.byId(this.id).hide() -- no longer seems needed with 'promise' architecture introduced in 1.8
			registry.byId(this.id).destroy();
			this.inherited(arguments);
		},
		
		onClose: function(){
//			registry.byId(this.id).hide() -- no longer seems needed with 'promise' architecture introduced in 1.8
			registry.byId(this.id).destroy();
			this.inherited(arguments);
		},
		onCancel: function(){
//			registry.byId(this.id).hide() -- no longer seems needed with 'promise' architecture introduced in 1.8
			registry.byId(this.id).destroy();
			this.inherited(arguments);
		},
		

		_handleDialogLaunch: function () {

			var launchParms = this._args.launchParms;
			var menuItem = this._args.menuItem;
			var context = this._args.context;

			var dialogContentWidget = null, diijtHeight;
			if (launchParms) {
				// Handle undefined path and dijit, use utils.DefaultConfirmationDialogTask
				if (!launchParms.modulePath && !launchParms.dijit) {
					launchParms.modulePath = "ibm.usmi.tasks";
					launchParms.dijit = "DefaultConfirmationDialogTask";
				}
				dialogContentWidget = launchParms.modulePath + "." + launchParms.dijit;

				// custom builds don't like passing variables into dojo.require
				// so we have to do a little trick instead - RFB
				// dojo.require(dialogContentWidget); // RFB
				dojo["require"](dialogContentWidget);
				var Classname = dojo.getObject(dialogContentWidget);
				
				if (this._dialogDijit) {
					this._dialogDijit.destroy();
					this._dialogDijit = null;
				}

				if (!this._dialogDijit) { //create the widget. 
					this._dialogDijit = new Classname({
						launchParms: launchParms,
						menuItem: menuItem,
						context: context
					});
					
					//add the new widget directly to the dialog object
					this.set("content", this._dialogDijit);
				}
				
				this.show();

				this._dialogDijit.startup();

				if (this._dialogDijit.resize) {
					if (dialogContentWidget === "ibm.usmi.scheduler.JobLaunchWizard") {
						var bb = domGeometry.position(this.containerNode);
						diijtHeight = bb.h;
						diijtHeight -= 32;
						this._dialogDijit.resize({
							w: 700,
							h: diijtHeight
						});
					} else {
						this._dialogDijit.resize();
					}
				}
	
				if (this._dialogDijit.updateUI) {
					// no longer need to call updateUI twice
					// this._dialogDijit.updateUI();
					this._dialogDijit.updateUI();
				}
				
	
				//this.layout()		-- no longer part of dijit/Dialog



			} else {
				if (this._detailsSelectorLogger.isError()) {
					this._detailsSelectorLogger.error("No launchParams specified for this task.  Cannot construct dialog content", "_handleDialogLaunch");
				}
			}

		},

		_getBodyHeight: function () {
			return ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight);
		}
	});
});
