/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/ResourceNavigator", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/topic",
	"dijit/_WidgetBase",
	"dijit/layout/ContentPane",
	"dijit/layout/BorderContainer",
	"dijit/registry",
	// TWL imports
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/navmodel/Dataset",
	"ibm/usmi/util/DefaultMessageHandler",
	"ibm/tivoli/tip/navwidget/dijit/NavWidget",
	// NextGen imports
	"ibm/usmi/widgets/table/DefaultItemsTableView",
	"ibm/usmi/util/BaseWidget",
	"ibm/usmi/widgets/util/ResourceNavigatorDAHelper",
	"ibm/usmi/widgets/util/DefaultContextMenuProvider",
	"ibm/usmi/widgets/util/ResourceMenuProviderHelper",
	"ibm/usmi/widgets/util/Constants",
	"ibm/usmi/core/utils/Logger",
	"dojo/i18n!./nls/CommonWidgetsResources"
], function (dojo, declare, lang, topic, _WidgetBase, ContentPane, BorderContainer, registry, TCTTable, Dataset,
		DefaultMessageHandler, NavWidget, DefaultItemsTableView, BaseWidget, ResourceNavigatorDAHelper,
		DefaultContextMenuProvider, ResourceMenuProviderHelper, Constants, Logger, mri) {

	return declare([
		_WidgetBase,
		BaseWidget,
		BorderContainer
	], {

		// attr that can be set html tag
		navigatorWidgetDefinitionId : "",

		// boolean: navigator stand alone, used to deterimine if should load dataset or wait and it will be set
		// by another means, ie. TreeSelector
		standAlone : true,

		// cmProvider: Custom context menu provider (must be provided on construction to be used)
		cmProvider : null,

		// Defaults
		DEFAULT_COLLECTION : "RESOURCE_NAVIGATOR",

		//attribute to disable retarget listening
		noRetarget : null,

		///////////////////////////////////////////////////////////////////
		// private members
		///////////////////////////////////////////////////////////////////

		//resource bundle
		_resources : mri,

		// the logger
		_resourceNavigatorLogger : new Logger("ibm.usmi.widgets", "ResourceNavigator"),

		// load up the task utils for the different types of task launch
		_taskUtil : null,

		// handle to TWL navigator
		_navWidget : null,

		// TODO -  This is tied to launching the navigator from action.
		_dataSetExternally : false,

		// dataset object
		_dataset : null,

		defaultTaskUtil : null,
		defaultActionHelper : null,
		_subscriptions : null,

		_createCMProvider : function () {
			// summary:
			//        Coming soon
			// description:
			//        Coming soon

			//Create the menu provider that we'll pass off to the pluggable views
			if (!this.cmProvider) {
				var helper = new ResourceMenuProviderHelper();
				this.cmProvider = new DefaultContextMenuProvider({
					view : this,
					getMenuItemsCallback : lang.hitch(helper, helper.getMenuItems),
					getMenuCommandCallback : lang.hitch(helper, helper.getMenuCommand)
				});
			}

		},

		constructor : function (/*constructor_kwArgs*/kwArgs) {
			// summary:
			//        Constructs PropertyNotebook object.

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "constructor");
			}

			this.title = this._resources.ResourceNavigatorTitle;
			this._subscriptions = [];
			if (!this.collectionID) {
				var targetInfo = this.getInitArg(Constants.TARGET_INFO);
				if (targetInfo) {
					this.collectionID = targetInfo[Constants.COLLECTION];
				}
			}

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "constructor");
			}
		},

		//look at these two functions and do things in a standard way
		startup : function () {
			this.ready();
		},

		ready : function () {
			console.log("Resource Navigator Ready!");
			this.makeNavWidget();
			if (this._dataSetExternally) {
				this._updateTargets(this._dataset, null);
				this._dataSetExternally = false;
			}

			//provide noRetarget option so users can turn off listening for retarget events
			if (!this.noRetarget) {
				var sub = topic.subscribe(Constants.TOPIC_RETARGET_INFO, lang.hitch(this, this.retargeted));
				this._subscriptions.push(sub);
			}
		},

		postCreate : function () {
			var datasetArg = this.getInitArg("dataset");
			if (datasetArg) {
				this.collectionID = datasetArg;
			}
			if (!this.collectionID) {
				this.standAlone = false;
			}

			this._subscriptions.push(dojo.subscribe(Constants.TOPIC_HASH_CHANGE, this, this.hashChange));
		},

		//Create the items table view
		makeNavWidget : function () {
			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "makeNavWidget");
			}

			// create a default context menu provider
			this.defaultActionHelper = new ResourceNavigatorDAHelper();
			// widget def on URl Overrides all
			defOnUrl = this.getUrlParmsObj().navigatorWidgetDefinitionId;

			if (defOnUrl) {
				this.navigatorWidgetDefinitionId = defOnUrl;
			} // check to see if it was specified as attribute
			else if (!this.navigatorWidgetDefinitionId) {
				this.navigatorWidgetDefinitionId = "UsmiNavigatorView-0.1.0";
			}

			// make sure cmProvider has been constructed
			this._createCMProvider();

			// id is passed in via data-dojo-props
			this._navWidget = new DefaultItemsTableView({
				id : "_navWidget" + this.id,
				'class' : 'interiorSpacious',
				height : "98%",
				region: "center",
				//homeDataset: dataset,
				daHelper : this.defaultActionHelper,
				//            widgetDefinitionId: this.navigatorWidgetDefinitionId,
				cmProvider : this.cmProvider
			});

			this.addChild(this._navWidget);

			//see if we were passed a dataset at construction
			if (this.getInitArg(Constants.TARGET_INFO) && this.getInitArg(Constants.TARGET_INFO).dataset) {
				this._dataset = this.getInitArg(Constants.TARGET_INFO).dataset;
				this._navWidget.setDataset(this._dataset, null, null);
				this.setTitle(this._dataset);
			} else {
				// We need to go ahead and return now to keep the main UI thread active
				// We'll schedule this for later
				//this.loadDataset();
				setTimeout(lang.hitch(this, this.loadDataset), 0);
			}

			this.inherited(arguments);

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "makeNavWidget");
			}
		},

		loadDataset : function () {
			// summary:
			//            Create the dataset for the table, only called if collectionID is provided on URL or 
			//			  as attr, otherwise assumed will be set later by selector.

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "loadDataset");
			}
			this._resourceNavigatorLogger.finer("this.collectionID: " + this.collectionID);
			var _processDataSet;
			// Check to make sure we were given provider, collection information
			if (!this.providerID) {
				// load default
				this.providerID = Constants.DEFAULT_PROVIDER;
			}

			if (!this.collectionID) {
				// load default
				this.collectionID = this.DEFAULT_COLLECTION;
			}

			//var myNavWidget = registry.byId("myNavWidget");
			_processDataSet = function (dataset) {
				this._navWidget.setDataset(dataset, null, null);
				this.setTitle(dataset);

				//Listen for when the rendered dataset changes
				dojo
						.connect(this._navWidget, Constants.SET_DATASET, lang.hitch(this,
								this._navWidgetSetDatasetHandler));

				//Listen for selection changes on nav widget
				dojo.connect(this._navWidget.selection, Constants.SELECTION_CHANGE, lang.hitch(this,
						this._navWidgetSelectionChangedHandler));
			};

			// can't blindly passdown sourceToken, if the sourceToken is null the create doesn't
			// include it on the rest call, but subsequent getTasks call do, which causes multiple
			// navcollections on the server side to be created.
			if (this.sourceToken != null) {
				this._dataset = new Dataset({
					onComplete : lang.hitch(this, _processDataSet),
					onError : function (error, exception, ioArgs) {
						var msgHandler = new DefaultMessageHandler({});
						msgHandler.displayException(exception);
					},
					baseUrl : Constants.BASE_REST_URL,
					provider : this.providerID,
					datasource : "",
					dataset : this.collectionID,
					parameters : {
						sourceToken : this.sourceToken
					},
					defaultProperties : []
				});
			} else {
				this._dataset = new Dataset({
					onComplete : lang.hitch(this, _processDataSet),
					onError : function (error, exception, ioArgs) {
						var msgHandler = new DefaultMessageHandler({});
						msgHandler.displayException(exception);
					},
					baseUrl : Constants.BASE_REST_URL,
					provider : this.providerID,
					datasource : "",
					dataset : this.collectionID,
					parameters : {},
					defaultProperties : []
				});
			}

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "loadDataset");
			}
		},

		resize : function () {
			if(this._navWidget) {
				if(this._navWidget.get("_table")) {
					this._navWidget.get("_table").resize();
				}
			}
			this.inherited(arguments);
		},
		
		//Destroy the topology canvas (largely so server will get notified to clean itself up)
		destroy : function () {
			// summary:
			//        Clean up any objects the Resource Navigator is hanging onto
			//Tell finder to clear context selection
			this.publishSelectionToFinder(null);

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "destroy");
			}

			if (this._dataset) {
				this._dataset.destroy();
			}
			if (this.cmProvider) {
				this.cmProvider.destroy();
			}
			if (this._navWidget) {
				this._navWidget.destroyRecursive();
			}
			if (this.defaultTaskUtil) {
				this.defaultTaskUtil.destroy();
			}
			if (this._subscriptions) {
				var i = 0;
				for (; i < this._subscriptions.length; i++) {
					this._subscriptions[i].remove();
				}
				this._subscriptions = null;
			}

			this.inherited(arguments);

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "destroy");
			}
		},

		hashChange : function (newHash) {
			if (this._resourceNavigatorLogger.isFiner()) {
				this._resourceNavigatorLogger.finer("hashchange called!" + dojo.hash(), "hashChange");

			}
			var urlParms = newHash.split('&');
			var idx = 0;
			var retargetedParms = {};
			for (; idx < urlParms.length; idx++) {
				var paramValues = urlParms[idx].split("=");
				if (paramValues[0] === 'dataset') {
					retargetedParms.collectionID = paramValues[1];
				} else if (paramValues[0] === 'workAreaDijit') {
					if (paramValues[1] === "ibm/usmi/widgets/ResourceNavigator") {
						retargetedParms.taskSelection = paramValues[1];
					}
				}

			}
			if (retargetedParms.collectionID && retargetedParms.taskSelection) {
				this.retargeted(retargetedParms);
			}

		},

		setDataset : function (/*ibm.tivoli.navmodel.Dataset*/dataset) {
			this._dataset = dataset;
			this._dataSetExternally = true;

		},

		//handler for dataset is set
		_navWidgetSetDatasetHandler : function (/*ibm.tivoli.navmodel.Dataset*/dataset) {
			// summary:
			//        Coming soon
			// description:
			//        Coming soon
			// dataset:
			//
			if (dataset) {
				if (this._resourceNavigatorLogger.isFiner()) {
					this._resourceNavigatorLogger.finer("DatasetHandler called: dataset = " + dataset.id,
							"_navWidgetSetDatasetHandler");
				}
			} else {
				if (this._resourceNavigatorLogger.isFiner()) {
					this._resourceNavigatorLogger.finer("DatasetHandler called dataset is null",
							"_navWidgetSetDatasetHandler"); // Randy added message for this condition
				}
			}
		},

		//handler for selection changes
		_navWidgetSelectionChangedHandler : function (objectContext) {
			// summary:
			//        Coming soon
			// description:
			//        Coming soon
			// objectContext:
			//

			var items = objectContext.getItems();
			//Tell finder to add context search for tasks for current selection
			this.publishSelectionToFinder(items);
			if (items) {
				if (items.length > 0) {
					if (this._resourceNavigatorLogger.isFiner()) {
						this._resourceNavigatorLogger.finer("NavWidget first selected item's id is = "
								+ items[0].getId(), "_navWidgetSelectionChangedHandler");
					}
				}
			}
			//            navWidget.hideBusyPane();

		},
		publishSelectionToFinder : function (items) {
			var selectedItems = "", i = 0;
			if (items) {
				for (; i < items.length; i++) {
					if (selectedItems !== "") {
						selectedItems += ",";
					}
					selectedItems += items[i].getId();
				}
			}
			topic.publish("CurrentResourceSelection", selectedItems);
		},

		_updateTargets : function (/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.Item[]*/
		items) {
			// summary:
			//       update based on retarget event being fired
			// description:
			//        Coming soon
			// dataset:

			//TODO: consider if we need to accomodate the _navWidget not really being there yet
			//save off the dataset and check for it when we're ready if we do
			this._navWidget.setDataset(dataset, null, null);
		},
		retargeted : function (retargetInfo) {
			if (retargetInfo.taskSelection === "ibm/usmi/widgets/ResourceNavigator") {
				if (retargetInfo.dataset) {
					if (this._dataset) {
						this._dataset.destroy(); //destroy old dataset
					}

					// set to new one
					this._dataset = retargetInfo.dataset;
					this._navWidget.setDataset(this._dataset, null, null);
					this.setTitle(this._dataset);
				}
				//no dataset included. Handle collectionId or sourcetoken setup
				else if (retargetInfo.sourceToken || retargetInfo.collectionID) {
					this.sourceToken = retargetInfo.sourceToken;
					this.collectionID = retargetInfo.collectionID;
					if (this._dataset) {
						this._dataset.destroy(); //destroy old dataset
					}
					this.loadDataset();
				}
			}
		},
		setTitle : function (dataset) {
			if (!this.notUpdateTitle) {
				dojo.publish(Constants.TOPIC_TASK_TITLE_CHANGE, {
					resource : dataset._dataset.label
				});
			}
		}
	});
});