/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultItemsTableView", [
	"dojo/_base/declare",
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/connect",
	"dojo/aspect",
	"dojo/date",
	"dijit/place",
	"dijit/registry",
	"dojo/i18n!./nls/ColumnChooser",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/tivoli/tip/navwidget/table/dijit/ItemsTableView",
	"ibm/usmi/widgets/table/ConfigureColumnsDialog",	
	"ibm/usmi/core/utils/Logger",
	"ibm/usmi/util/DefaultMessageHandler",
	"./DefaultNavModelTable",
	"ibm/usmi/widgets/table/DefaultTablePersistenceMixin"
], function (declare, dojo, lang, array, connect, aspect, date, place, registry, ConfigureColumnsMRI, FrameworkMRI, ItemsTableView, ConfigureColumnsDialog,
		Logger, DefaultMessageHandler, DefaultNavModelTable, DefaultTablePersistenceMixin) {

	return declare([ ItemsTableView, DefaultTablePersistenceMixin
	], {

		// the logger
		_defaultItemsTableViewLogger : new Logger("ibm.usmi.widgets.table", "DefaultItemsTableView"),
		_countsChangedHandle : null,
		_counts : null,
		_msg: null,
		_origSummaryMsg: null,

		// 190751 Vars START		
		// These numbers are chosen by tests. Other values might not work.
		_DELAYTIME : 3000,
		_isStoreBusy : false, // the table data store is refreshing	
		_refreshTimerActive: false, // indicates whether onRefreshTimer is active
		_refreshStart : 0,	// start time of a refresh
		_refreshStartOld : 0, // start time of previous refresh
		_refreshEnd : 0, // end time of a refresh
		_estimatedRefresh : 3000, // estimated refresh time from the last complete one
		_estimatedThrottle : 10000,
		_timeLastUpdate : null,
		_currentColumns : null,
		// 190751 Vars END

		constructor : function () {
			this._pDEFAULT_WIDGET_DEF_INFO = {
				className : "ibm.tivoli.navmodel.TableWidgetDef",
				baseUrl : "/ibm/tivoli/rest",
				provider : "any",
				datasource : "any",
				widget : "table",
				definition : "UsmiItemsTable"
			};
			this._pTableClassName="ibm.usmi.widgets.table.DefaultNavModelTable";
		},

		augmentGridParams : function (gridParams) {
			// For info on gridParams, look at ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView
			// and also at dojoe/table/Table.
			// Look at the source; do not trust the comments.
			this.inherited(arguments);

			gridParams.plugins.nestedSorting = false;

			// Make it scroll by default, instead of paging
			gridParams.plugins.pagination = false;

			// Get rid of the checkbox column for selection
			gridParams.plugins.indirectSelection = false;

			// Get rid of the refresh menu option
			gridParams.toolbarOptions.refreshMenu = false;

			// Get rid of the printer menu option
			// gridParams.toolbarOptions.printerMenu = false;

			// Get rid of the export menu option
			// gridParams.toolbarOptions.exporterMenu = false;

			// Disable drag-and-drop of rows and columns
			gridParams.plugins.dnd = false;

			// set row selection to multiple, could also be done in widgetdef
			gridParams.selectionMode = "multi";
			
			gridParams = this.persistenceGridParams(this._dataStore, gridParams);

			return gridParams;
		},
		
		_onRefreshTimer : function() {
			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.finest(this._refreshStart+" onRefreshTimer: "+this._isStoreBusy, "_onRefreshTimer");
			}							
		
			// still busy; so reschedule the timer
			if (this._isStoreBusy) {
				this._estimatedRefresh = this._DELAYTIME;
				setTimeout(lang.hitch(this, this._onRefreshTimer), this._estimatedRefresh);				
			}
			// no longer busy; so kickoff another onRefresh
			else if (this._table && this._table._grid && this._table._grid.model && this._table._grid.model.store) {
					this._timeLastUpdate = new Date();
					this._table._grid.model.store.onRefresh();					
			}
		},		

		// AbstractTableView._initTableBilder connects to onTableLoad, and inadvertently shows the filter bar.
		// This aspect is called afterward, and hides the filter bar again.
		_initTableBuilder : function () {
			//TODO, hack, memory leak on NavModelTable.js, luckily we can clean up here

			var toolbarId, configureTableButtonId, configureTableButton, actionMenu, menuItems, lastMenuItem;
			if (this._table && this._table._tooltip) {
				this._table._tooltip.destroy();
			}
			
			// Fix defect 204191
			// We've seen cases where if you have filtered on a table (tree selector), if you step out of the table and then
			// back in to the table, _initTableBuilder is called twice for the table.  The first time into this code block,
			// the dataset in the datastore is different from the dataset we're now using for the table (so we're switching
			// tables).  We're then immediately called a second time with the same dataset (so the dataset in the datastore
			// matches the dataset we're using for our table).  In this case, we'll skip calling inherited (which destroys 
			// the table and causes other problems) -- on the other hand, if the user has configured columns we might need to
			// rebuild the table....
			// ...so, if we don't already have a datastore (or the datstore we have doesn't already have a dataset), OR
			// the ID of dataStore.dataset doesn't match the ID of the dataset we're now working on) -- rebuild
			if (((!this._dataStore) || (!this._dataStore.dataset)) || (this._dataStore.dataset.getId() !== this.dataset.getId()))			{
				// We need to call inherited to destroy the existing table and build the new one....
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("*******************requested to build a new table.****************************", "_initTableBuilder");				
				}														
				this.inherited(arguments);
				_currentColumns = this._tableWidgetDefinition.getDataSetColumns();	// store away the columns currently being displayed
			} else {
				// this appears to be the case where we're told to build the same table, we're already displaying.
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("*******************Requested to rebuild the table we're currently displaying -- checking to see if it's necessary.****************************", "_initTableBuilder");				
				}							

				// We'll want to skip doing this unless we are in here because the user has used configured columns -- and has changed the columns being displayed.  
				var rebuildNecessary = false;  // assume no rebuild is necessary
				
				// Verify we have enough information to determine if a rebuild is necessary....
				if ((!_currentColumns) || (!this._tableWidgetDefinition) || (!this._tableWidgetDefinition.getDataSetColumns()))  {
					// we don't have information to determine if we can skip rebuild...therefore, force a rebuild.
					if (this._defaultItemsTableViewLogger.isFinest()) {
						this._defaultItemsTableViewLogger.finest("*******************There is not enough information to determine if we can skip the rebuild...so force it****************************", "_initTableBuilder");				
					}												
					rebuildNecessary = true;
				} 
				
				if (!rebuildNecessary) {
					// First check to see if the number of columns has changed by retrieving them from the table widget definition
					var updatedColumns = this._tableWidgetDefinition.getDataSetColumns();	// the columns we want to display
					var numUpdatedColumns = updatedColumns.length;							// number of columns we want to display
				
					if (_currentColumns.length !== numUpdatedColumns) {
						// the number of columns we want to display is different from the number of columns we're currently displaying.  We want to force the table to be rebuilt.
						if (this._defaultItemsTableViewLogger.isFinest()) {
							this._defaultItemsTableViewLogger.finest("*******************The number of columns we're currently displaying has changed.  Force the table to be rebuilt.****************************", "_initTableBuilder");				
						}												
						rebuildNecessary = true;
					} else {
						// the number of columns hasn't changed, but the user could have changed the column order, or did an add/remove of things.  If that change has happened, 
						// then we need to rebuild the table.  Loop through the colums comparing the IDs.  At the first difference, we want to force the table to be rebuilt.
						for(var i = 0; i < numUpdatedColumns; i++){
							if (_currentColumns[i].id !== updatedColumns[i].id) {
								if (this._defaultItemsTableViewLogger.isFinest()) {
									this._defaultItemsTableViewLogger.finest("*******************It appears as though the order of the column IDs has changed.  Force the table to be rebuilt.****************************", "_initTableBuilder");				
								}				
								rebuildNecessary = true;
								break;
							}
						}	// end for loop					
					}	// end the number of columns is the same	
				}
				
				if (!rebuildNecessary) {
					// IF we still don't believe a rebuild of the table is needed, we could be in the special case where the table has
					// an additional filtering switch (event log and active alerts).  First see the dataset IDs match
					
					// we have confidence that we have the _dataStore/_dataStore.dataset/_datastore.dataset.getID -- otherwise we 
					// would have identified the necessity of a rebuild before we got into this else-block
					var oldID = this._dataStore.dataset.getId();

					// we have confidence we have the dataset.getID -- otherwise we would have identified the necessity of a rebuild 
					// before we got into this else-block
					var newID = this.dataset.getId();
					
					// If the IDs match -- we need to check to see if the table is one of the special tables that have an additional filtering switch					
					if (oldID === newID) {
						// Check to see if this is the Event Log table
						if (oldID === "EVENT_LOG") {
							// Retrieve the event filter IDs
							oldFilterID = this._dataStore.dataset.getArgs().parameters.FILTER_ID;
							newFilterID = this.dataset.getArgs().parameters.FILTER_ID;
						
							// If the event filter IDs are different -- we must rebuild
							if (oldFilterID !== newFilterID) {
								rebuildNecessary = true;
							}					
						} // end we're displaying the event log				
						
						// If the IDs indicate we're displaying Active Alerts
						if (oldID=== "ACTIVE_ALERTS") {
							// Retrieve the active alerts category filter IDs
							oldCatID = this._dataStore.dataset.getArgs().parameters.CATEGORY;
							newCatID = this.dataset.getArgs().parameters.CATEGORY;
						
							// If the active alerts category filter IDs are different -- we must rebuild
							if (oldCatID !== newCatID) {
								rebuildNecessary = true;
							}					
						}	// end we're displaying Active Alerts														

						// if the IDs indicate we're working with compute nodes, check the source tokens
						if (oldID === "NEXT_GEN_COMPUTE_NODE_VIEW") {
							// Retrieve the source token
							oldSourceToken = this._dataStore.dataset.getArgs().parameters.sourceToken;
							newSourceToken = this.dataset.getArgs().parameters.sourceToken;
						
							// If the active alerts category filter IDs are different -- we must rebuild
							if (oldSourceToken !== newSourceToken) {
								rebuildNecessary = true;
							}											
						}	// end we're displaying compute nodes
						
					}	// end the IDs match
				} 	// end we still don't believe a rebuild is necessary
					
				if (rebuildNecessary) {
					this.inherited(arguments);			// rebuild the table
					_currentColumns = this._tableWidgetDefinition.getDataSetColumns();  // save away the new list of columns currently being displayed
				} else {
					// rebuild is unnecessary -- we decided we don't want to force the table to be rebuilt.
					if (this._defaultItemsTableViewLogger.isFinest()) {
						this._defaultItemsTableViewLogger.finest("*******************The rebuild of the table does not appear to be necessary.  We will bypass this step.****************************", "_initTableBuilder");				
					}							
				}	// end rebuild is unnecessary
				
			}	// end we were told to rebuild the same table we're already displaying.	
			
		
			aspect.after(this._table, "onTableLoad", lang.hitch(this, function () {
				if (this._table) {
					this._table.setFilter(null, true);

					//@188636 START
					this.initPersistence(this, this._grid, this._table._toolbar._quickFilter.searchField);
					//@188636 END
				}
			}));

			//Hack in our column chooser
			if (this._table && this._table._toolbar) {
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("setting the custom Column Chooser", "_initTableBuilder");
				}
				this._table._toolbar._configureTable = function () {
					this.configureTableDialog = new ConfigureColumnsDialog({
						grid : this.inGrid,
						_structure : this._origStructure
					});
					this.configureTableDialog.show();
				};
				toolbarId = this._table._toolbar.id;
				if (toolbarId) {
					configureTableButtonId = "configureColIcon" + toolbarId;
					configureTableButton = registry.byId(configureTableButtonId);
					if (configureTableButton) {
						configureTableButton.set("title", ConfigureColumnsMRI.CONFIG_COLUMNS);
					}
				}
				if (this._table._toolbar.actionListWidget) {
					actionMenu = this._table._toolbar.actionListWidget.dropDown;
					if (actionMenu) {
						menuItems = actionMenu.getChildren();
						if (menuItems && menuItems.length > 0) {
							lastMenuItem = menuItems[menuItems.length - 1];
							if (lastMenuItem && lastMenuItem.iconClass === "configureColIcon") {
								lastMenuItem.set("title", ConfigureColumnsMRI.CONFIG_COLUMNS);
								lastMenuItem.set("label", ConfigureColumnsMRI.CONFIG_COLUMNS);
							}
						}
					}
				}

			}
			
			
			//need to change the total items in filter bar message to the dataset items count
			if(this._table && this._table._grid && this._table._grid.filterBar){
				aspect.before(this._table._grid.filterBar, "_buildFilterState", lang.hitch(this, function () {
					this._table._grid.filterBar._totalSize=this.dataset.getItemCount();
					//need to show the filter bar if it's not showed
					if(!this._table._grid.filterBar.isVisible()){
						this._table._grid.filterBar.show();
					}
				}));
			}
			
			
			if(this._table && this._table._grid && this._table._grid && this._table._grid.model){
				connect.connect(this._table._grid.model, "onNew", lang.hitch(this, this.refreshFilterBar));
				connect.connect(this._table._grid.model, "onDelete", lang.hitch(this, this.refreshFilterBar));
				connect.connect(this._table._grid.model, "onSet", lang.hitch(this, this.refreshFilterBar));

				if (this._table._grid.bar.plugins.bottom[0][0]) { //this._table._grid.summaryBar) {
					connect.connect(this._table._grid.bar.plugins.bottom[0][0], "refresh", lang.hitch(this, this.refreshSummaryBar));
				}
			}
			
			//
			// 190751 START				
			//
			// This is to fix the performance issue of the grid always showing busy when a constant change happens on a sorted column.
			// When a sorted column is changing, the grid requires a refresh on the entire data set, which takes quite a long time.
			// During the refresh, if there are multiple new change events on the same column cames in, multiple refresh will be submitted.
			// If the changing keeps in a constant speed that is too fast for one refresh to complete, the grid will show busy all the time.
			// The fix is to stretch out multiple refresh requests and seperate them with increased time slots, if these requests come in
			// during a refresh is in progress.
			// Because the code for handling events are deep in TWL. We can't stop refresh(onRefresh) been called directly,
			// which is also called somewhere else.
			// A trick here is to hijack _handleItemPropertiesChangedEvent() of DatasetItemStore, even if we have to copy some code over.
			// 
			
			if (this._table && this._table._grid && this._table._grid.model && this._table._grid.model.store) {
			
				// Hook up onRefresh, so the busy flag is set before the refresh starts.
				aspect.around(this._table._grid.model.store, "onRefresh", lang.hitch(this, function(orginalFunc) {					
					return lang.hitch(this, function(){
						// mark busy; when table is rendered; will be set back to false
						this._isStoreBusy = true;
						this._refreshStart = (new Date()).getTime();						
						return orginalFunc();						
					});
				}));
				
				
				// Hook up event handling function in DatasetItemStore, so we can stop refreshing when the store is too busy.
				// Most code are copied from the orginal function.
				aspect.around(this._table._grid.model.store, "_handleItemPropertiesChangedEvent", lang.hitch(this, function(orginalFunc) {
					return lang.hitch(this, function(event) {
						var _this = this._table._grid.model.store;	// Keep the context right.
						var items = event.getItems();    	
						var propsArray = event.getProperties();			
						if (items && propsArray) {
							var refresh = false;
							var length = items.length;			
							for (var index = 0;  index < length;  ++index) {
								var properties = propsArray[index];				
								if (properties) {
									var propsLength = properties.length;
									for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
										var item = items[index];
										var property = properties[propIndex];
										var propId = property.getId();
										if (_this._trace.isFinest()) {
											_this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
										}
										
										_this.onSet(item, propId, null, property.getDisplayValue());
										
										if (!refresh && (_this._affectsSort(propId) || _this._affectsCondition(propId))) {
											refresh = true;							
										}
									}										
								}				
							}
							
							if (refresh) {								
								// New stuff, if the store is busy, schedule a refresh						
								if (this._isStoreBusy) {
									// only start timer if one is not currently active
									if (!this._refreshTimerActive) {
										this._refreshTimerActive = true;
										setTimeout(lang.hitch(this, this._onRefreshTimer), 
											this._estimatedRefresh);
									}
								} else {
									if(!this._timeLastUpdate){
										//no previous refresh run so update
										this._refreshTimerActive = true;
										_this.onRefresh();
									}else{
										// start timer if one is not currently active to throttle down the updates.  
										if (!this._refreshTimerActive) {
											this._refreshTimerActive = true;
											setTimeout(lang.hitch(this, this._onRefreshTimer), 
												this._estimatedThrottle+5);
										}
									}
									
								}
							}	
						}						
					});
				}));	
					
			}
			
			if (this._table && this._table._grid && this._table._grid.body) {				
				connect.connect(this._table._grid.body, "onRender", lang.hitch(this, function() {
					if (this._isStoreBusy && this._refreshStart>0) {
						// It seems onRender will be called multiple times for one refresh.
						if (this._refreshStart === this._refreshStartOld) {
							if (this._defaultItemsTableViewLogger.isFinest()) {
								this._defaultItemsTableViewLogger.finest(this._refreshStart+" onRender: duplicated ", "_initTableBuilder");
							}							
						} else {
							// When the render is done, the store should be OK to take another refresh
							this._refreshStartOld = this._refreshStart;
							this._refreshEnd = (new Date()).getTime();	
							this._estimatedRefresh = this._refreshEnd-this._refreshStart;
							if (this._defaultItemsTableViewLogger.isFinest()) {
								this._defaultItemsTableViewLogger.finest(
									this._refreshStart+" onRender(end, duration): "+this._refreshEnd+", "+this._estimatedRefresh, 
									"_initTableBuilder");
							}							
							this._timeLastUpdate = new Date();
							this._isStoreBusy = false;							
							this._refreshTimerActive = false;
						}
					}
				}));
			}					
			//			
			// 190751 END
			//			
			
		},	
		
		refreshFilterBar: function(){
			if(this._table && this._table._grid && this._table._grid && this._table._grid.model && this._table._grid.filterBar){
				if(!this._table._grid.filterBar.isVisible()){
					//if filter bar is not visible, we do not want to refresh it
					return;
				}
				this._table._grid.filterBar._currentSize = this._table._grid.model.size();
				this._table._grid.filterBar._buildFilterState();
			}
		},

		refreshSummaryBar: function(caller) {
			var summaryBar = this._table._grid.bar.plugins.bottom[0][0];
			if (caller !== 'onCountsChanged') {
				this._origSummaryMsg = summaryBar.domNode.innerHTML;
			}

			if (this._msg && this._counts) {
				var selectedCount = 0;
				var rowIds = this._table._grid.select.row.getSelected();
				if (rowIds) {
					selectedCount = rowIds.length;
				}
				var summary = lang.replace(FrameworkMRI["HybridSummaryCounts"], [this._counts.totalCount, this._counts.filteredCount, this._counts.availableCount, selectedCount]);
				summaryBar.domNode.innerHTML = summary;
			}
			else {
				summaryBar.domNode.innerHTML = this._origSummaryMsg;	
			}		
		},		

		_cleanUpImagePath : function (imagePath) {
			return this.inherited(arguments);
		},		

		handleLinkClickAction : function (viewId, objectId, propId, objectType) {
			// summary:
			//        Method to override base implementation so we get notified and can
			//      handle when a link is clicked in a table cell
			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.entering("", "handleLinkClickAction");
				this._defaultItemsTableViewLogger.finest("handling link click for table view - prop: " + propId,
						"handleLinkClickAction");
			}
			var object, property;

			//ask the dataset for the correct object based on the objects id.
			//since we have already rendered these items, the value should be
			//cached and should be quick.
			switch (objectType) {
			case "ibm.tivoli.navmodel.Item":
				object = this.dataset.getItem({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Link":
				object = this.dataset.getLink({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Relationship":
				object = this.dataset.getRelationship({
					itemId : objectId
				});
				break;
			default:
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("unknown objectType: " + objectType,
							"handleLinkClickAction");
				}
			}

			if (object) {
				property = object.getProperty(propId);
			}

			if (object && property) {
				this.daHelper.executeDefaultAction(this, property);
			}

			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.exiting("", "handleLinkClickAction");
			}
		},

		// Override getContextMenuAdapter, so we can reposition the context menu.
		// Necessary since the menu will change size after it is first displayed,
		// as more items come in over CURI.		
		getContextMenuAdapter : function () {
			// get the ContextMenuAdapter, which I will later return.
			var cma = this.inherited(arguments);
			var coords;

			// Before the ContextMenuAdapter.contextMenuOpen method
			aspect.before(cma, "contextMenuOpen", function (event) {
				// Save the original mouse-click coordinates for later
				coords = {
					x : event.pageX,
					y : event.pageY
				};
			});

			// After the ContextMenuAdapter._menuItemsLoaded method
			aspect.after(cma, "_menuItemsLoaded", function (success) {
				// Put the menu in the correct place, so it does not go off the page
				if (success) {
					var menu = this.getMenu();
					var orient = menu.isLeftToRight() ? 'L' : 'R';
					var wrapper = menu.domNode.parentNode;
					if (wrapper !== null && coords) {
						place.at(wrapper, coords, orient === 'R' ? [
							'TR',
							'BR',
							'TL',
							'BL'
						] : [
							'TL',
							'BL',
							'TR',
							'BR'
						]);
					}
				}
			}, true);

			return cma;
		},

		_onCountsChanged: function(args) {			
			if (this._msg) {
				this._msg.destroy();
				this._msg = null;
				this._counts = null;
			}
				
			if (args.availableCount) {
				var msgHandler = new DefaultMessageHandler({});
				var msg = lang.replace(FrameworkMRI["HybridDataset"], [args.totalCount, args.filteredCount, args.availableCount]);
				var desc = FrameworkMRI["HybridDatasetDescription"];				
				this._msg = msgHandler.displayMessage("", msgHandler.INFO, msg, desc, false);
				this._counts = args;				
			}

			this.refreshSummaryBar('onCountsChanged');
		},
		
		_createTableStore: function(columnIds){				
			// disconnect from previous datastore
			if (this._countsChangedHandle) {
				this._countsChangedHandle.remove();
			}
			
			this.inherited(arguments);

			// connect to new datastore
			if (this._dataStore && this._dataStore.onCountsChanged) {
				this._countsChangedHandle = connect.connect(this._dataStore, "onCountsChanged", lang.hitch(this, this._onCountsChanged));
			}
		},		

		destroy : function () {
			//TODO, hack, memory leak on NavModelTable.js, luckily we can clean up here
			if (this._table && this._table._tooltip) {
				this._table._tooltip.destroy();
				this._table._tooltip = null;
			}
			if (this._table && this._table._tooltipProperties) {
				this._table._tooltipProperties.destroyRecursive();
				this._table._tooltipProperties = null;
			}
			if (this._countsChangedHandle) {
				this._countsChangedHandle.remove();
			}
			this.inherited(arguments);
		}

	});
});
