/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
        "ibm/usmi/widgets/table/DefaultNavModelTable", [
            "dojo/_base/declare",
            "dojo/_base/array",
            "ibm/tivoli/tip/navwidget/table/dijit/NavModelTable",
            "ibm/usmi/core/utils/Logger"
        ],
        function(declare, array, NavModelTable, Logger) {
            return declare(
                    "ibm.usmi.widgets.table.DefaultNavModelTable",
                    [ NavModelTable
                    ],
                    {
                        _logger: new Logger("ibm.usmi.widgets.table", "DefaultNavModelTable"),

                        _customTooltip: function(/*Object*/target) {
                            var statusCatPat = /^STATUS_CAT:/, content, contentArray = [], imgSrc, 
                                item = this._rolloverTarget, methodName = "_customTooltip", logger = this._logger, 
                                property = this._rolloverCellTarget, propId, propVal, propTooltip, tooltipArray, 
                                tooltipArrayLength, lastTooltipEntry, ellipsis = /\.\.\.$/, isEndwithEllipsis;
                            if (!item) {
                                logger.finest("item is not hovered", methodName);
                                return this.inherited(arguments);
                            }
                            if (!property) {
                                logger.finest("property is not hovered", methodName);
                                return this.inherited(arguments);
                            }

                            propId = property.getId();
                            if (!propId) {
                                logger.finest("property id is null or undefined", methodName);
                                return this.inherited(arguments);
                            }

                            //test if it's a status category property
                            if (!statusCatPat.test(propId)) {
                                logger.finest("property id " + propId + " is not a status category id", methodName);
                                return this.inherited(arguments);
                            }

                            propTooltip = property.getTooltip();

                            if (!propTooltip) {
                                logger.finest("tooltip is not set", methodName);
                                return this.inherited(arguments);
                            }

                            tooltipArray = propTooltip.split(",");
                            tooltipArrayLength = tooltipArray.length;
                            if (tooltipArrayLength === 0) {
                                logger.finest("tooltip array length is zero", methodName);
                                return this.inherited(arguments);
                            }

                            propVal = property.getValue();
                            if (!propVal) {
                                logger.warn("property value is null or undefined", methodName);
                                return this.inherited(arguments);
                            }
                            imgSrc = propVal.getImage();
                            if (!imgSrc) {
                                logger.error("status property image is null or undefined", methodName);
                                return this.inherited(arguments);
                            }

                            lastTooltipEntry = tooltipArray[tooltipArrayLength - 1];
                            isEndwithEllipsis = ellipsis.test(lastTooltipEntry);
                            if (isEndwithEllipsis) {
                                lastTooltipEntry = lastTooltipEntry.substring(0, lastTooltipEntry.length - 3);
                                tooltipArray[tooltipArrayLength - 1] = lastTooltipEntry;
                            }

                            contentArray.push("<table>");
                            array.forEach(tooltipArray, function(item) {
                                contentArray.push("<tr><td>");
                                contentArray.push("<img src=\"");
                                contentArray.push(imgSrc);
                                contentArray.push("\"/><span>");
                                contentArray.push(item);
                                contentArray.push("</span></td></tr>");
                            });

                            if (isEndwithEllipsis) {
                                contentArray.push("<tr><td>...</td</tr>");
                            }

                            contentArray.push("</table>");
                            content = contentArray.join("");
                            if (logger.isFinest()) {
                                logger.finest("customTooltip is " + content, methodName);
                            }
                            return content;
                        }

                    });
        });
