/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultTablePersistenceMixin", [
    "dojo/_base/declare",
    "dojo/_base/kernel",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/aspect",
    "dijit/registry",
    "ibm/usmi/widgets/table/DefaultItemsTableViewPersist"
], function(declare, dojo, lang, array, aspect, registry, Persist) {

    return declare([], {
        // summary:
        //      Mixin utility methods for working with persistence
        // description:
        //      Defines utility methods for initializing persistence for a table          

        // add the gridparams necessary for persistence
        persistenceGridParams: function(datastore, gridParams) {
            var persistId, module;
            if (!gridParams.id) {
                if (datastore && datastore.dataset && datastore.dataset._dataset) {
                    persistId = "persistgrid:" + datastore.dataset._dataset.id;
                } else {
                    persistId = "emptygrid";
                }
            } else {
                persistId = gridParams.id;
            }

            module = {
                moduleClass: Persist,
                key: this._buildPersistenceKey(persistId)
            };
            if (gridParams.modules) {
                gridParams.modules.push(module);
            } else {
                gridParams.modules = [ module
                ];
            }
            return gridParams;
        },

        // hook up persistence
        initPersistence: function(t, g, searchField) {
            var selection, filter, persist, result, model, selectIndex, index;

            // check parameters for whether row selection should be persisted...
            if (g.persist) {
                filter = g.persist.registerAndLoad('filter', function() {
                    if (g.filterBar) {
                        return g.filterBar.filterData;
                    } else {
                        return null;
                    }
                });

                persist = g.persist;
                selection = g.persist.registerAndLoad('selection', function() {
                    if (g.persist && g.select) {
                        return g.select.row.getSelected();
                    } else {
                        // The persist module will do two saves on tree tables, one on grid.destroy
                        // and one on window unload.  The latter will happen after grid.destroy, so 
                        // the data we're trying to save is invalid.  So just return the persisted value...
                        result = persist.get(persist.key);
                        if (result) {
                            return result.selection;
                        } else {
                            return null;
                        }
                    }
                });

                if (filter && g.filterBar) {
                    // if it is a single basic filter, insert text into quick filter field
                    if (searchField && filter.type === "any" && filter.conditions && filter.conditions.length === 1
                            && filter.conditions[0].condition === "contain") {
                        searchField.value = filter.conditions[0].value;
                    }
                    g.filterBar.applyFilter(filter);
                    g.filterBar.show(); // ensure filter bar displays
                }

                if (selection) {
                    model = g.model;
                    array.forEach(selection, function(id) {
                        model.markById(id, 1);
                    });

                    // scroll and onSelected need to be delayed until the model data appears
                    setTimeout(function() {
                        // Need to select the row this way so the context menu works
                        // after the row is highlighted
                        if (g.select.row) {
                            array.forEach(selection, function(id) {
                                var index = model.idToIndex(id);
                                g.select.row.selectByIndex(index);  
                            });
                        }

                        if (g.vScroller) {
                            selectIndex = -1;
                            array.forEach(selection, function(id) {
                                index = model.idToIndex(id);
                                if (selectIndex === -1 || selectIndex > index) {
                                    selectIndex = index;
                                }
                            });
                            g.vScroller.scrollToRow(selectIndex);
                        }
                    }, 1000);
                }
            }
        },

        // build a new persistence key based on the url and sorted hash parameters
        // strip out sourceToken and other variables that artificially make the key unique for table
        _buildPersistenceKey: function(id) {
            var hash = dojo.hash(), i = 0, newHash = "";

            // build up an array so we can sort it
            var storedParams = [];
            var params = hash.split('&');
            for (i = 0; i < params.length; i++) {
                if (params[i].split('=')[0] !== "sourceToken") {
                    storedParams.push(params[i]);
                }
            }

            // sort it...
            storedParams.sort();
            for (i = 0; i < storedParams.length; i++) {
                if (i > 0) {
                    newHash += "&";
                }
                newHash += storedParams[i];
            }

            return window.location.protocol + "//" + window.location.host + window.location.pathname + "?" + newHash + '/' + id;
        }

    });
});